#include <xc.h>
#include <stdlib.h>

#define _XTAL_FREQ 8000000
#define THRAESHOLD   0x30

// コンフィギュレーション１の設定
#pragma config FOSC     = INTOSC   // 内部ｸﾛｯｸ使用する(INTOSC)
#pragma config WDTE     = OFF      // ｳｵｯﾁﾄﾞｯｸﾞﾀｲﾏｰ無し(OFF)
#pragma config PWRTE    = ON       // 電源ONから64ms後にﾌﾟﾛｸﾞﾗﾑを開始する(ON)
#pragma config MCLRE    = OFF      // 外部ﾘｾｯﾄ信号は使用せずにﾃﾞｼﾞﾀﾙ入力(RA3)ﾋﾟﾝとする(OFF)
#pragma config CP       = OFF      // ﾌﾟﾛｸﾞﾗﾑﾒﾓﾘｰを保護しない(OFF)
#pragma config CPD      = OFF      // ﾃﾞｰﾀﾒﾓﾘｰを保護しない(OFF)
#pragma config BOREN    = ON       // 電源電圧降下常時監視機能ON(ON)
#pragma config CLKOUTEN = OFF      // CLKOUTﾋﾟﾝをRA4ﾋﾟﾝで使用する(OFF)
#pragma config IESO     = OFF      // 外部・内部ｸﾛｯｸの切替えでの起動はなし(OFF)
#pragma config FCMEN    = OFF      // 外部ｸﾛｯｸ監視しない(OFF)

// コンフィギュレーション２の設定
#pragma config WRT    = OFF        // Flashﾒﾓﾘｰを保護しない(OFF)
#pragma config PLLEN  = OFF        // 動作クロックを32MHzでは動作させない(OFF)
#pragma config STVREN = ON         // スタックがオーバフローやアンダーフローしたらリセットをする(ON)
#pragma config BORV   = HI         // 電源電圧降下常時監視電圧(2.5V)設定(HI)
#pragma config LVP    = OFF        // 低電圧プログラミング機能使用しない(OFF)

unsigned int sense();

// メインの処理
void main()
{
    unsigned int cap;
     // ＵＳＡＲＴ機能の設定を行う
     RXDTSEL = 0 ;             // 2番ピン(RA5)をＲＸ受信ピンとする
     TXCKSEL = 0 ;             // 3番ピン(RA4)をＴＸ送信ピンとする
     TXSTA  = 0b00100100 ;     // 送信情報設定：非同期モード　８ビット・ノンパリティ
     RCSTA  = 0b10010000 ;     // 受信情報設定
     SPBRG  = 51 ;             // ボーレートを９６００(高速モード)に設定


     OSCCON  = 0b01110010 ; // 内部クロックは8ＭＨｚとする
     ANSELA  = 0b00000100 ; // アナログはAN2を使用し、残りをすべてデジタルI/Oに割当
     TRISA   = 0b00000100 ; // AN2(RA2/CPS2)だけ入力その他のピンは出力に割当てる(RA3は入力専用)
     PORTA   = 0b00000000 ; // 出力ピンの初期化(全てLOWにする)
     // 容量検知モジュール(ＣＰＳＭ)の設定
     CPSCON0 = 0b00001000 ; // オシレータは中範囲(中速の発信周波数)で利用する
     CPSCON1 = 0b00000010 ; // CPS2から入力する
     // タイマー１の設定
     T1CON   = 0b11000001 ; // 容量検知オシレータでTIMER1をｶｳﾝﾄする、ﾌﾟﾘｽｹｰﾗｶｳﾝﾄ値 1:1
     TMR1H   = 0 ;          // タイマー1の初期化
     TMR1L   = 0 ;
     PEIE    = 1 ;          // 周辺装置割り込みを許可する
     GIE     = 1 ;          // 全割り込み処理を許可する

     CPSON   = 1 ;          // 容量検知モジュール開始

     while(1) {
         cap = sense();
         int on = 0;

          if (cap < THRAESHOLD) {
              on = 1;
          }
          else {
              on = 0;
          }
         RA5 = on;

          while(TXIF == 0) ;     // 送信可能になるまで待つ    *1)
          TXREG = cap >> 8 ;          // 送信する
          while(TXIF == 0) ;     // 送信可能になるまで待つ    *1)
          TXREG = cap & 0xff ;          // 送信する
          while(TXIF == 0) ;     // 送信可能になるまで待つ    *1)
          TXREG = on;          // 送信する
     }
}

unsigned int sense() {
    unsigned int cap_sum = 0;
    int i;
    for (i = 0; i < 10; i++) {
        CPSON = 1 ;                 // 容量検知モジュール開始
        __delay_ms(1);
        CPSON = 0 ;                 // 容量検知モジュール停止

        cap_sum += (TMR1H << 8) + TMR1L ; // カウント値を読み込む
        TMR1H = 0 ;                 // タイマー1の初期化
        TMR1L = 0 ;
    }
    return cap_sum / 10;
}